package net.bigeon.gclc.process;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import net.bigeon.gclc.process.mocks.TaskMock;

public class TaskPoolTest {

    public TaskMock task = new TaskMock();
    public TaskPool pool = new TaskPool(false);

    @Test
    public void testTaskPool() {
        assertFalse("task pool should not auto clear if specified",
                pool.isAutoClearing());
        pool = new TaskPool();
        assertTrue("Default task pool should not auto clear", pool.isAutoClearing());
    }

    @Test
    public void testAdd() {
        final int id = pool.add(task);
        assertTrue("Added task pid should be in pool", pool.getPIDs().contains(id));
        assertEquals("Added task should be in pool", task, pool.get(id));
    }

    @Test
    public void testGet() {
        assertNull("Inexistent task id should provide null as task in pool",
                pool.get(60));
        final int id = pool.add(task);
        assertEquals("Added task should be in pool at its id", task, pool.get(id));
        assertNull("Added task should be in pool at its id", pool.get(id + 1));
    }

    @Test
    public void testGetPIDs() {
        assertTrue("new pool should be empty", pool.getPIDs().isEmpty());
        final int id = pool.add(task);
        assertNull("Added task should be in pool at its id", pool.get(id + 1));
        assertEquals(
                "Pool task addition should change the pid collection size accordingly", 1,
                pool.getPIDs().size());
    }

    @Test
    public void testRemove() {
        assertTrue("new pool should be empty", pool.getPIDs().isEmpty());
        pool.remove(60);
        assertTrue("removal of inexistent task should be NOP", pool.getPIDs().isEmpty());
        final int id = pool.add(task);
        assertEquals(
                "Pool task addition should change the pid collection size accordingly", 1,
                pool.getPIDs().size());
        pool.remove(id + 1);
        assertEquals("removal of inexistent task should be NOP", 1,
                pool.getPIDs().size());
        pool.remove(id);
        assertTrue("removal of task should reduce the number of tasks",
                pool.getPIDs().isEmpty());
    }

    @Test
    public void testShutdown() {
        final int id = pool.add(task);
        task.reset();
        pool.shutdown();
        assertEquals("Shutdown should request shut down of tasks, only once", 1,
                task.getNumberSetRunning());
    }

}
