/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.manager.ConsoleInput.java
 * Created on: Nov 13, 2017
 */
package net.bigeon.gclc.manager;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.util.function.Supplier;

/** A console application input.
 *
 * @author Emmanuel Bigeon */
public interface ConsoleInput extends AutoCloseable {

    /** Closes the input.
     *
     * @throws IOException if the close raised an exception */
    @Override
    void close() throws IOException;

    /** Get the prompt string.
     *
     * @return the prompt prefix */
    Supplier<String> getPrompt();

    /** Indicate to the input that is should interrompt the prompting, if possible.
     * <p>
     * The pending {@link #prompt()} or {@link #prompt(String)} operations should
     * return immediately by throwing an java.io.InterruptedIOException. */
    void interruptPrompt();

    /** Test if the input is closed.
     * <p>
     * If this is true, {@link #prompt()} methods will return immediatly and a null
     * chain.
     *
     * @return if the manager is closed. */
    boolean isClosed();

    /** Prompt the user.
     *
     * @return the user inputed string
     * @throws IOException if the manager is closed or could not read the prompt
     * @throws java.io.InterruptedIOException if the prompt was interrupted */
    String prompt() throws IOException;

    /** Prompt the user, with an allotated time to answer.
     *
     * @param timeout the time to wait in milliseconds
     * @return the user inputed string, null if the timeout was reached
     * @throws IOException if the manager is closed or could not read the prompt
     * @throws java.io.InterruptedIOException if the prompt was interrupted */
    String prompt(long timeout) throws IOException;

    /** Prompt the user, with a hint on what is prompted.
     *
     * @param message the message to prompt the user
     * @return the user inputed string
     * @throws IOException if the manager is closed or could not read the prompt
     * @throws java.io.InterruptedIOException if the prompt was interrupted */
    String prompt(String message) throws IOException;

    /** Prompt the user, with a hint on what is prompted and an allotated time to
     * answer.
     *
     * @param timeout the time to wait in milliseconds
     * @param message the message to prompt the user
     * @return the user inputed string, null if the timeout was reached
     * @throws IOException if the manager is closed or could not read the prompt
     * @throws java.io.InterruptedIOException if the prompt was interrupted */
    String prompt(String message, long timeout) throws IOException;

    /** Set a prompting prefix.
     *
     * @param prompt the prompt */
    void setPrompt(String prompt);

    /** Set the prompting string.
     *
     * @param string the prompt string */
    void setPrompt(Supplier<String> string);
}
